	list p=12F509
#include <p12F509.inc>

	__CONFIG	( _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC )

; watchdog timer on
;	__CONFIG	( _MCLRE_OFF & _CP_OFF & _WDT_ON & _IntRC_OSC )

	cblock 0x08
Id
BitCnt
CurData

; Impulsion Data
ImpulsionCnt	; impulsion count
BitRegister		; the LS-bit determines whether the impulsion is 0 or 1
CycleCnt		; cycle counter
	endc


#define	LED_bit	GPIO,2
#define IR_bit GPIO,4
#define BTN_bit GPIO,0
#define SYNC_bit GPIO,1		; synchronization pin for oscilloscopes
							; debug use only

#define TRANSMIT_HIGH BitRegister,0

#define HUMAN_ID	7		; 0-7

if HUMAN_ID == 0
#define GAP_DURATION  .31
#define DEVICE_ID b'111000'
endif

if HUMAN_ID == 1
#define GAP_DURATION  .37
#define DEVICE_ID b'110001'
endif

if HUMAN_ID == 2
#define GAP_DURATION  .41
#define DEVICE_ID b'101010'
endif

if HUMAN_ID == 3
#define GAP_DURATION  .43
#define DEVICE_ID b'100011'
endif

if HUMAN_ID == 4
#define GAP_DURATION  .47
#define DEVICE_ID b'011100'
endif

if HUMAN_ID == 5
#define GAP_DURATION  .53
#define DEVICE_ID b'010101'
endif

if HUMAN_ID == 6
#define GAP_DURATION  .59
#define DEVICE_ID b'001110'
endif

if HUMAN_ID == 7
#define GAP_DURATION  .61
#define DEVICE_ID b'000111'
endif


ImpulsionWM macro	val
	MOVWF ImpulsionCnt
	if val == 0
		BCF TRANSMIT_HIGH
	else
		BSF TRANSMIT_HIGH
	endif
	CALL Impulsion
	endm

ImpulsionM	macro	len, val
	MOVLW len
	ImpulsionWM val
	endm

XmitWM macro
	MOVWF CurData
	CALL XmitByte
	endm

	org 0
Start
	BCF LED_bit
	MOVLW b'00000001'		; initialize I/O pins, only GP0 as input
	TRIS GPIO	; as output

	MOVLW b'10001000' ; pull-ups enabled
	OPTION

	MOVLW DEVICE_ID		; device ID
	MOVWF Id			; to variable Id

FrameLoop
	BSF LED_bit			; LED on while transmitting

	BSF SYNC_bit		; sync start for oscilloscope output

	ImpulsionM 8, 1		; frame start HHHHHHHH
	ImpulsionM 4, 0		; frame start LLLL

	BCF SYNC_bit		; sync stop for oscilloscope output

	MOVFW Id			; first byte:
	XmitWM				; transmit ID

	;MOVFW Id			; second byte
	;XORLW 0xFF			; transmit
	;XmitWM				; not ID

	ImpulsionM 4, 1		; frame stop marker HHHH

	BCF LED_bit			; LED off at EOT

	MOVLW GAP_DURATION
	MOVWF CycleCnt
Gap_FrameLoop
	MOVLW .33			; 51 cycles of 3 * 256 microseconds = 39168 microseconds
	MOVWF CurData
Gap_51Loop
	MOVLW 0xFF
	MOVWF BitCnt
Gap_ByteLoop
	DECFSZ BitCnt
	GOTO Gap_ByteLoop
	DECFSZ CurData
	GOTO Gap_51Loop
	DECFSZ CycleCnt
	GOTO Gap_FrameLoop


	BTFSS BTN_bit		; if "test" mode
	INCF Id				; increment Id

	GOTO FrameLoop



XmitByte
	MOVLW 6				; 6-bit words
	MOVWF BitCnt
ByteLoop
	MOVLW 1
	MOVWF ImpulsionCnt
	BSF TRANSMIT_HIGH
	CALL Impulsion
	
	MOVLW 1
	MOVWF ImpulsionCnt
	BTFSC CurData,5
	BSF ImpulsionCnt,1
	BCF TRANSMIT_HIGH
	CALL Impulsion

	RLF CurData
	DECFSZ BitCnt
	GOTO ByteLoop
	RETLW 0
		

Impulsion
	MOVLW .22
	MOVWF CycleCnt

CycleLoop				; Loop CycleCnt times here
	BTFSC TRANSMIT_HIGH
	BSF IR_bit
	NOP
	NOP
	NOP
	NOP
;	NOP
	NOP
	NOP
	NOP
	BCF IR_bit
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	DECFSZ CycleCnt
	GOTO CycleLoop

	DECFSZ ImpulsionCnt
	GOTO Impulsion

	RETLW 0
		
		
	End
